@@ -1,5 +1,12 @@
 import { loadImJoyBasicApp } from "imjoy-core/dist/imjoy-loader";
 import { setupRPC } from "imjoy-core/dist/imjoy-rpc";
+import {
+  IFileBrowserFactory
+} from '@jupyterlab/filebrowser';
+import {
+  showErrorMessage,
+} from '@jupyterlab/apputils';
+import { PathExt } from '@jupyterlab/coreutils';
 
 import { ContentsManager } from '@jupyterlab/services';
 
@@ -48,6 +55,49 @@ if not hasattr(ipykernel, 'connect'):
   }
 }
 
+async function parseURL(app, browser){
+  const queryString = window.location.search;
+  const urlParams = new URLSearchParams(queryString);
+  const urls = urlParams.getAll("load");
+  let lastFile = null;
+  for(let url of urls){
+    let type = '';
+    let blob;
+    // fetch the file from the URL
+    try {
+      const req = await fetch(url);
+      blob = await req.blob();
+      type = req.headers.get('Content-Type') ?? '';
+    } catch (reason) {
+      if (reason.response && reason.response.status !== 200) {
+        reason.message = trans.__('Could not open URL: %1', url);
+      }
+      return showErrorMessage(trans.__('Cannot fetch'), reason);
+    }
+    // upload the content of the file to the server
+    try {
+      const name = PathExt.basename(url).split("?")[0];
+      const file = new File([blob], name, { type });
+      const model = await browser.model.upload(file);
+      lastFile = model.path
+      console.log("File uploaded: " + model.path);
+    } catch (error) {
+      return showErrorMessage(
+        trans._p('showErrorMessage', 'Upload Error'),
+        error
+      );
+    }
+  }
+  let file2Open = urlParams.get("open") 
+  if(file2Open === "1" ) file2Open = lastFile;
+  if(file2Open){
+    setTimeout(()=>{
+      app.commands.execute('docmanager:open', {
+        path: file2Open
+      });
+    }, 0)
+  }
+}
 class ImjoyExtension {
   constructor(jupyterBaseUrl) {
     this.baseUrl = jupyterBaseUrl;
@@ -212,8 +262,10 @@ class ImjoyExtension {
 
 export default {
   id: 'imjoy-jupyterlab-extension',
+  requires: [IFileBrowserFactory],
   autoStart: true,
-  activate: function (app) {
+  activate: async function (app, factory) {
+    const { defaultBrowser: browser } = factory;
     const jupyterBaseUrl =
     app.serviceManager.settings.serverSettings.baseUrl;
     app.docRegistry.addWidgetExtension(
@@ -223,5 +275,8 @@ export default {
     console.log(
       'JupyterLab extension imjoy-jupyterlab-extension is activated!'
     );
+
+    await parseURL(app, browser);
+    
   }
 }
